//+------------------------------------------------------------------+
//|                                               SimpleBreakOut.mq5 |
//|                        Copyright 2015, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2016, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#property version   "1.00"
//--- input parameters
input int      RangeLength=20;   // -  
input int      TicksForEnter=30; // -   
input int      BarsForExit=10;   //   N 
//input double   TP_range=0.5;     // TP  % 
//input double   SL_range=1.0;     // SL  % 
input double   Lots=0.1;         //    
//---
#include <Trade\Trade.mqh>
#include <Trade\PositionInfo.mqh>
//---  
CTrade trade;                    //   
CPositionInfo positioninfo;      //    
double UpLevel;                  //   
double DnLevel;                  //   
datetime LastEnterTime;          //     
datetime LastExitTime;           //     
datetime BreakTime;              //  
int BreakDirection;              //  
int ticksUp, ticksDn;            // -       
int tick_counter=0;              //  
double lastbid=-1;               //     Bid
int UpBreak,DnBreak;
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//---      
   trade.LogLevel(LOG_LEVEL_ERRORS);
//---     ,       
   LastEnterTime=0; //       
//---    
   BreakTime=0;
   UpBreak=0;
   DnBreak=0;
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---
   PrintFormat("Break Up=%d,  Break Down=%d",UpBreak,DnBreak);
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//---     
   if(isNewBar())
     {
      SetRangeLevels(RangeLength);
     }
//---   ?      
   if(isBreakOut())
     {
      //---     
      StartTickCount(OpenTimeCurrentBar());
     }
//---    
   if(PositionOpened())
     {
      //---     
      CheckForExit(BarsForExit);
     }
   else //  
     {
      //---   ?
      CheckForEnter();
     }
//---     
  }
//+------------------------------------------------------------------+
//|  true                                     |
//+------------------------------------------------------------------+
bool isNewBar()
  {
//---        
   datetime static open_time=0;
//---       
   datetime time=OpenTimeCurrentBar();
//---       
   if(open_time==0)
     {
      open_time=time;
      return (true);
     }
   else
     {
      if(open_time==time)
        {
         //---   ,     ()
         return (false);
        }
      else
        {
         //---          ()
         open_time=time;
         //---      ,    
         BreakTime=0;
         tick_counter=0;
        }
     }
//---      
   return true;
  }
//+------------------------------------------------------------------+
//|                                   |
//+------------------------------------------------------------------+
bool PositionOpened()
  {
//---     true
   return positioninfo.Select(_Symbol);
  }
//+------------------------------------------------------------------+
//|                       |
//+------------------------------------------------------------------+
void SetRangeLevels(int bars)
  {
   double high[],low[];
//---   High  Low   
   CopyHigh(_Symbol,_Period,1,bars,high);
   CopyLow(_Symbol,_Period,1,bars,low);
//---           
   UpLevel=high[(int)ArrayMaximum(high)];
   DnLevel=low[(int)ArrayMinimum(low)];
//---
  }
//+------------------------------------------------------------------+
//|  ,    -               |
//+------------------------------------------------------------------+
void CheckForExit(int bars_for_hold)
  {
//---      
   datetime open_time=positioninfo.Time();
   datetime time_bars_for_hold=OpenTimeByIndex(bars_for_hold);
//---       =bars_for_hold
   if(open_time<time_bars_for_hold)
     {
      //---  
      Print(StringFormat("Position %s",positioninfo.TypeDescription()));
      if(!trade.PositionClose(_Symbol))
        {
         //---       
         trade.PrintRequest();
        }
      else
        {
         //---    
         trade.PrintResult();
         LastExitTime=TimeCurrent();
        }
     }
//---
  }
//+------------------------------------------------------------------+
//|  true                      |
//+------------------------------------------------------------------+
bool isBreakOut()
  {
//---    
   datetime time=OpenTimeCurrentBar();
   if(BreakTime==time)
      return false; //      
//---     
   MqlTick tick;
   SymbolInfoTick(_Symbol,tick);
//---    Bid      
   if(tick.bid>UpLevel)
     {
      //---  
      BreakDirection=1;
      UpBreak++;
      return (true);
     }
   if(tick.bid<DnLevel)
     {
      //---  
      BreakDirection=-1;
      DnBreak++;
      return (true);
     }
//---   
   BreakDirection=0;
   return (false);
  }
//+------------------------------------------------------------------+
//|                                         |
//+------------------------------------------------------------------+
void CheckForEnter()
  {
//---    
   datetime open_time=OpenTimeCurrentBar();
   if(open_time==LastEnterTime)
     {
      //---       , 
      return;
     }
//---       
   if(open_time==BreakTime)
     {
      //---       
      if(tick_counter<TicksForEnter)
        {
         //---            
         tick_counter++;
         //---     
         MqlTick tick;
         SymbolInfoTick(_Symbol,tick);
         //---    
         if(tick.bid>lastbid)
            ticksUp++;
         if(tick.bid<lastbid)
            ticksDn++;
        }
      else
      if(tick_counter>=TicksForEnter)
        {
         return;
        }
     }
   else //           ,   
     {
      return;
     }
//---       
   if(tick_counter<TicksForEnter)
      return; //    
//---    
   bool canEnter=false;
   if((BreakDirection==1) && (ticksUp-ticksDn>0))
      canEnter=true;
   if((BreakDirection==-1) && (ticksUp-ticksDn<0))
      canEnter=true;
//---         ,      
   if(!canEnter)
     {
      //tick_counter=0;
      //BreakTime=0;
      return;
     }

   MqlTick tick;
   SymbolInfoTick(_Symbol,tick);
//---  
   if(BreakDirection==1)
     {
      if(trade.Buy(Lots))
        {
         LastEnterTime=open_time;
        }
     }
//---  
   if(BreakDirection==-1)
     {
      if(trade.Sell(Lots))
        {
         LastEnterTime=open_time;
        }
     }

//---
  }
//+------------------------------------------------------------------+
//|                                                   |
//+------------------------------------------------------------------+
void StartTickCount(datetime time)
  {
//---        Bid
   if(BreakTime==0)
     {
      BreakTime=time;
      //---        
      MqlTick tick;
      SymbolInfoTick(_Symbol,tick);
      lastbid=tick.bid;
      //---        
      ticksUp=0;
      ticksDn=0;
     }
//---
  }
//+------------------------------------------------------------------+
//|                               |
//+------------------------------------------------------------------+
datetime OpenTimeCurrentBar()
  {
   datetime time[];
//---       i=0
   CopyTime(_Symbol,_Period,0,1,time);
   return (time[0]);
  }
//+------------------------------------------------------------------+
//|                               |
//+------------------------------------------------------------------+
datetime OpenTimeByIndex(int index)
  {
   datetime time[];
//---      i=index
   CopyTime(_Symbol,_Period,0,index,time);
   return (time[0]);
  }

//+------------------------------------------------------------------+
